#include <iostream>
#include <exception>
#include <string>
#include <fstream>
#include <algorithm>
#include <functional>

using namespace std;

struct ListElement
{
	ListElement()
		: data(0), pNext(NULL), pPrevious(NULL) {}

	ListElement(int i)
		: data(i), pNext(NULL), pPrevious(NULL) {}

	ListElement(int i, ListElement *pNext, ListElement *pPrevious)
		: data(i), pNext(pNext), pPrevious(pPrevious) {}

	int data;
	ListElement *pNext;
	ListElement *pPrevious;
};

enum Mode { NONE = 0, INC = 1, DEC = 2 };

class LinkedList
{
public:
	LinkedList() : head(NULL), tail(NULL), count(0), mode(NONE) { cout << "constructor QIs" << endl; }
	LinkedList(const LinkedList&);

	~LinkedList() { cout << "destructor QIs" << endl; }

	void addTail(int i);
	void delTail();

	void addHead(int i);
	void delHead();

	void insert(int i, int position);
	void remove(int position);

	void autoInsert(int);

	void print();
	void rprint();
	void clear();

	Mode getMode() { return mode; }
	void setMode(int);

	void readFile(fstream &input);
	void writeFile(fstream &output);

	int getCount() { return count; };

private:
	bool checkAndCreateHead(int);
	void output(ostream &output);

	void insertInc(int);
	void insertDec(int);

	template <typename COMP_FUNC>
	int getAutoInsertPosition(int i, COMP_FUNC &compare);

	ListElement *head;
	ListElement *tail;
	int count;
	Mode mode;
};

int main()
{
	LinkedList list;

	list.addHead(3);
	list.addTail(10);
	list.insert(7, 2);

	cout << "ثeƵƬO " << list.getCount() << endl;

	list.delHead();
	list.delTail();
	list.remove(0);

	cout << "ثeƵƬO " << list.getCount() << endl;


	/*
	int choice, i, p;
	LinkedList ll;
	string filename;
	fstream file;

	do
	{
		cout << "ثe@ " << ll.getCount() << "ơAثeҦO : ";

		switch (ll.getMode())
		{
		case NONE:
			cout << "@Ҧ" << endl;
			cout << "==================================================" << endl;
			cout << " 1. sWƦܧ  2. sWƦY  3. sWƦܬYB" << endl;
			break;
		case INC:
			cout << "WҦ" << endl;
			cout << "==================================================" << endl;
			cout << " 0. ۰ʴJ" << endl;
			break;
		case DEC:
			cout << "Ҧ" << endl;
			cout << "==================================================" << endl;
			cout << " 0. ۰ʴJ" << endl;
			break;
		}

		cout << " 4. R  5. RY  6. RY@" << endl
			 << " 7. CXҦ  8. ϦCҦ  9. MҦ" << endl
			 << "10. XƦɮ 11. qɮŪJ 12. ܼҦ" << endl
			 << "13. {" << endl;

		cout << "> ";
		cin >> choice;

		try
		{
			switch (choice)
			{
			// sWƦܧ
			case 1:
				cout << "nJƦrO : ";
				cin >> i;

				ll.addTail(i);
				break;

			// sWƦY
			case 2:
				cout << "nJƦrO : ";
				cin >> i;

				ll.addHead(i);
				break;

			// sWƦܬYB
			case 3:
				cout << "nJƦrO : ";
				cin >> i;

				cout << "nJmO : ";
				cin >> p;

				ll.insert(i, p);

				break;

			// R
			case 4:
				ll.delTail();
				break;


			// RY
			case 5:
				ll.delHead();
				break;


			// RY@
			case 6:
				cout << "nR : ";
				cin >> p;

				ll.remove(p);
				break;

			// CXҦ
			case 7:
				ll.print();
				break;

			// ϦCҦ
			case 8:
				ll.rprint();
				break;


			// MҦ
			case 9:
				ll.clear();
				break;

			// XƦɮ
			case 10:
				cout << "пJɦW : ";
				cin >> filename;
				file.open(filename.c_str(), ios::out);

				if (!file.is_open())
					throw exception("ɮ׶}ҥ");
				
				ll.writeFile(file);
				file.close();
				break;

			// qɮŪJ
			case 11:
				cout << "пJɦW : ";
				cin >> filename;
				file.open(filename.c_str(), ios::in);

				if (!file.is_open())
					throw exception("ɮ׶}ҥ");

				ll.readFile(file);
				file.close();
				break;

			// ܼҦ
			case 12:
				cout << "0.@Ҧ 1.WҦ 2.Ҧ : ";
				cin >> i;
				if (i < 0 || i > 2) throw "J~";
				ll.setMode(i);
				break;

			case 0:
				cout << "nJƦrO : ";
				cin >> i;
				ll.autoInsert(i);
				break;

			case 13:
				break;

			default:
				throw exception("J~");
			}
			cout << endl;
			system("pause");
			system("cls");
		}
		catch (exception &e)
		{
			cout << "ERROR : " << e.what() << endl;
		}
	}while (choice != 13);

	return 0;
	*/
}

#if 0
LinkedList::LinkedList(const LinkedList &ll)
	: head(NULL), tail(NULL), count(0), mode(NONE)
{
	ListElement *tmp = ll.head;

	while (tmp != NULL)
	{
		this->addTail(tmp->data);
		tmp = tmp->pNext;
	}
}

LinkedList::~LinkedList()
{
	if (count != 0)
	{
		if (count == 1)
			delete head;
		else
		{
			ListElement *tmp = this->head->pNext;

			while (tmp != NULL && tmp->pNext != NULL)
			{
				delete tmp->pPrevious;
				tmp = tmp->pNext;
			}
			delete tmp;
		}
	}
}

bool LinkedList::checkAndCreateHead(int i)
{
	if (head != NULL)
		return false;

	head = tail = new ListElement(i);
	++count;
	return true;
}

void LinkedList::addTail(int i)
{
	if (checkAndCreateHead(i))
		return;

	ListElement *tmp = tail;
	tmp->pNext = new ListElement(i, NULL, tmp);
	tail = tmp->pNext;

	++count;
}

void LinkedList::delTail()
{
	if (count == 0)
		return;

	if (--count == 0)
	{
		delete head;
		head = tail = NULL;
		return;
	}

	ListElement *tmp = tail->pPrevious;
	delete tail;
	tail = tmp;
	tail->pNext = NULL;
}

void LinkedList::addHead(int i)
{
	if (checkAndCreateHead(i))
		return;

	ListElement *tmp = head;
	head = new ListElement(i, tmp, NULL);
	tmp->pPrevious = head;

	++count;
}

void LinkedList::delHead()
{
	if (count == 0)
		return;

	if (--count == 0)
	{
		delete head;
		head = tail = NULL;
		return;
	}

	ListElement *tmp = head->pNext;
	delete head;
	head = tmp;
	head->pPrevious = NULL;
}

void LinkedList::insert(int i, int position)
{
	if (position < 0 || position > count)
		throw std::out_of_range("position out of range");

	if (checkAndCreateHead(i))
		return;

	if (position == 0)
	{
		addHead(i);
		return;
	}

	if (position == count)
	{
		addTail(i);
		return;
	}

	ListElement *tmp = head;

	for (int j = 0; j < position - 1; ++j)
		tmp = tmp->pNext;

	ListElement *t2 = tmp->pNext;

	tmp->pNext = new ListElement(i, t2, tmp);
	t2->pPrevious = tmp->pNext;

	++count;
}

void LinkedList::remove(int position)
{
	if (position < 0 || position >= count)
		throw std::out_of_range("position out of range");

	if (count == 0)
		return;

	if (position == 0)
	{
		delHead();
		return;
	}

	if (position == count - 1)
	{
		delTail();
		return;
	}

	ListElement *tmp = head;

	for (int j = 0; j < position - 1; ++j)
		tmp = tmp->pNext;

	ListElement *t2 = tmp->pNext->pNext;
	delete tmp->pNext;
	tmp->pNext = t2;

	--count;
}

void LinkedList::print()
{
	output(cout);
}

void LinkedList::rprint()
{
	ListElement *tmp = tail;

	while (tmp != NULL)
	{
		cout << tmp->data << endl;
		tmp = tmp->pPrevious;
	}
}

void LinkedList::clear()
{
	if (count == 0) return;

	ListElement *tmp = tail->pPrevious;

	while (tmp != NULL)
	{
		delete tmp->pNext;
		tmp = tmp->pPrevious;
	}

	count = 0;
	head = tail = NULL;
}

void LinkedList::readFile(fstream &infile)
{
	int i;

	while (infile >> i)
		if (mode == NONE) addTail(i);
		else autoInsert(i);
}

void LinkedList::writeFile(fstream &o)
{
	output(o);
}

void LinkedList::output(ostream &o)
{
	ListElement *tmp = head;

	while (tmp != NULL)
	{
		o << tmp->data << endl;
		tmp = tmp->pNext;
	}
}

void LinkedList::setMode(int m)
{
	if (m == mode) return;

	switch (m)
	{
	case 0: mode = NONE; return;
	case 1: mode = INC;  break;
	case 2: mode = DEC;  break;
	}

	if (count == 0)
		return;

	LinkedList tmpll(*this);
	this->clear();

	ListElement *tmp = tmpll.head;

	while (tmp != NULL)
	{
		autoInsert(tmp->data);
		tmp = tmp->pNext;
	}
}

void LinkedList::autoInsert(int i)
{
	if (mode == NONE)
		throw exception("Ҧ䴩ާ@");

	if (mode == INC)
		insertInc(i);
	else
		insertDec(i);
}

void LinkedList::insertInc(int i)
{
	insert(i, getAutoInsertPosition(i, less<int>()));
}

void LinkedList::insertDec(int i)
{
	insert(i, getAutoInsertPosition(i, greater<int>()));
}

template <typename COMP_FUNC>
int LinkedList::getAutoInsertPosition(int i, COMP_FUNC &compare)
{
	ListElement *tmp = head;

	int t = 0;

	while (tmp != NULL)
	{
		if (compare(i, tmp->data))
			break;
		tmp = tmp->pNext;
		++t;
	}

	return t;
}
#endif

void LinkedList::addHead(int i)
{
	cout << "wsW " << i << " Y" << endl;
	++count;
}

void LinkedList::delHead()
{
	cout << "wRY" << endl;
	--count;
}

void LinkedList::addTail(int i)
{
	cout << "wsW " << i << " ܧ" << endl;
	++count;
}

void LinkedList::delTail()
{
	cout << "wR" << endl;
	--count;
}

void LinkedList::insert(int i, int position)
{
	cout << "wJ " << i << " ܦm " << position << endl;
	++count;
}

void LinkedList::remove(int position)
{
	cout << "wR " << position << " " << endl;
	--count;
}