#include <iostream>
#include <exception>

using namespace std;

struct ListElement
{
	ListElement()
		: data(0), pNext(NULL) {}

	ListElement(int i)
		: data(i), pNext(NULL) {}

	ListElement(int i, ListElement *pNext)
		: data(i), pNext(pNext) {}

	int data;
	ListElement *pNext;
};

class LinkedList
{
public:
	LinkedList() : root(NULL), count(0) {}
	LinkedList(const LinkedList&);
	~LinkedList();

	void addTail(int i);
	void addHead(int i);
	void clear();
	int  getCount() { return count; };

	int operator[](int i);

	LinkedList& operator=(const LinkedList &list);
	LinkedList& operator+=(const LinkedList &list);
	LinkedList& operator--();
	LinkedList operator-();
	LinkedList operator--(int);

	bool operator==(const LinkedList &list);

private:

	ListElement *root;
	int count;

	friend LinkedList operator+(const LinkedList &list1, const LinkedList &list2);
	friend ostream& operator<<(ostream &o, const LinkedList &list);
	friend istream& operator>>(istream &i, LinkedList &list);
};

LinkedList operator+(const LinkedList &list1, const LinkedList &list2)
{
	LinkedList tmp;

	ListElement *t = list1.root;

	while (t != NULL)
	{
		tmp.addTail(t->data);
		t = t->pNext;
	}

	t = list2.root;

	while (t != NULL)
	{
		tmp.addTail(t->data);
		t = t->pNext;
	}

	return tmp;
}

ostream& operator<<(ostream &o, const LinkedList &list)
{
	ListElement *t = list.root;

	while (t != NULL)
	{
		o << t->data << " ";
		t = t->pNext;
	}

	return o;
}

istream& operator>>(istream &i, LinkedList &list)
{
	int tmp;

	while (i >> tmp)
	{
		list.addTail(tmp);
	}

	i.clear();
	i.ignore();

	return i;
}

int main()
{
	LinkedList list1, list2;

	cout << "пJYzƦrAåH@ӫDƦrNJ" << endl;
	cout << "пJĤ@ list  : ";
	cin >> list1;

	cout << "пJĤG list  : ";
	cin >> list2;

	cout << "list1 : " << list1 << endl;
	cout << "list2 : " << list2 << endl;

	cout << "===== operator [] ============" << endl;

	for (int i = 0; i < list1.getCount(); i++)
		cout << "list[" << i << "] = " << list1[i] << endl;

	cout << "===== operator + =============" << endl;

	cout << "list3 = list1 + list2" << endl;
	LinkedList list3 = list1 + list2;
	cout << "list3 = " << list3 << endl;

	cout << "===== operator = =============" << endl;

	cout << " list2 = list1 " << endl;
	list2 = list1;

	cout << "list1 : " << list1 << endl;
	cout << "list2 : " << list2 << endl;

	cout << "list1 P list2 O_ۦP : " << (list1 == list2) << endl << endl;

	cout << "list1.addTail(10);" << endl;

	list1.addTail(10);

	cout << "list1 : " << list1 << endl;
	cout << "list2 : " << list2 << endl;

	cout << "list1 P list2 O_ۦP : " << (list1 == list2) << endl;

	cout << " list1 += list2" << endl;

	list1 += list2;

	cout << list1 << endl;

	cout << "===== operator - =============" << endl;
	cout << "-list1 : " << -list1 << endl;

	cout << "===== prefix operator -- =====" << endl;
	cout << "list1   : " << list1 << endl;
	cout << "--list1 : " << --list1 << endl;
	cout << "list1   : " << list1 << endl;

	cout << "===== postfix operator -- ====" << endl;
	cout << "list1   : " << list1 << endl;
	cout << "list1-- : " << list1-- << endl;
	cout << "list1   : " << list1 << endl;

	system("pause");
	return 0;
}

LinkedList::LinkedList(const LinkedList &ll)
	: root(NULL), count(0)
{
	ListElement *tmp = ll.root;

	while (tmp != NULL)
	{
		this->addTail(tmp->data);
		tmp = tmp->pNext;
	}
}

LinkedList::~LinkedList()
{
	this->clear();
}

void LinkedList::addTail(int i)
{
	if (root == NULL)
	{
		root = new ListElement(i);
	}
	else
	{
		ListElement *tmp = root;
		
		while (tmp->pNext != NULL)
		{
			tmp = tmp->pNext;
		}

		tmp->pNext = new ListElement(i, NULL);
	}
	++count;
}

void LinkedList::addHead(int i)
{
	if (root == NULL)
	{
		root = new ListElement(i);
	}
	else
	{
		ListElement *tmp = root;
		root = new ListElement(i, tmp);
	}

	++count;
}

void LinkedList::clear()
{
	if (count == 0) return;

	ListElement *tmp = root->pNext;

	while (tmp != NULL)
	{
		delete root;
		root = tmp;
		tmp = tmp->pNext;
	}
	delete root;
	root = NULL;
	count = 0;
}

LinkedList LinkedList::operator-()
{
	LinkedList tmp;

	ListElement *t = root;

	while (t != NULL)
	{
		tmp.addHead(t->data);
		t = t->pNext;
	}

	return tmp;
}

LinkedList& LinkedList::operator=(const LinkedList &list)
{
	this->clear();

	*this += list;

	return *this;
}

LinkedList& LinkedList::operator +=(const LinkedList &list)
{
	ListElement *tmp = list.root;

	while (tmp != NULL)
	{
		this->addTail(tmp->data);
		tmp = tmp->pNext;
	}

	return *this;
}

int LinkedList::operator [](int index)
{
	if (index < 0 && index >= this->count)
		throw std::out_of_range("operator[] out of range");

	ListElement *tmp = root;

	for (int i = 0; i < index; i++)
		tmp = tmp->pNext;

	return tmp->data;
}

bool LinkedList::operator ==(const LinkedList &list)
{
	if (this->count != list.count)
		return false;

	ListElement *tmp1 = this->root;
	ListElement *tmp2 = list.root;

	for (int i = 0; i < this->count; i++)
	{
		if (tmp1->data != tmp2->data)
			return false;

		tmp1 = tmp1->pNext;
		tmp2 = tmp2->pNext;
	}

	return true;
}

LinkedList& LinkedList::operator --()
{
	ListElement *tmp = root;

	while (tmp)
	{
		--tmp->data;
		tmp = tmp->pNext;
	}

	return *this;
}

LinkedList LinkedList::operator --(int)
{
	LinkedList tmp = (*this);

	--(*this);

	return tmp;
}